<?php /*
Plugin Name: Global Native Media Selector
Plugin URI: http://www.gnative.com
Description: This plugin is designed for developers to use to create a custom field that will emable multiple selections of media assets from the wordpress media library. It Supplies a list of functions that can be used within a wordpress theme
Version: 1.0.1
Author: Shane Saunders
Author URI: http://www.gnative.com/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/> */

//
if ( !defined('GNMS_VERSION') ) define( 'WPPT_VERSION', 0.2 );

/**
* Define wp-content and plugin urls/paths
*/
if ( ! defined( 'WP_CONTENT_URL' ) )
	define( 'WP_CONTENT_URL', get_option( 'siteurl' ) . '/wp-content' );
if ( ! defined( 'WP_CONTENT_DIR' ) )
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );
if ( ! defined( 'WP_PLUGIN_URL' ) )
	define( 'WP_PLUGIN_URL', WP_CONTENT_URL. '/plugins' );
if ( ! defined( 'WP_PLUGIN_DIR' ) )
	define( 'WP_PLUGIN_DIR', WP_CONTENT_DIR . '/plugins' );


include_once('gnms_views.php');

//
if ( !class_exists( "Gnms" ) ) {

	class Gnms {

		/**
		* @var string
		*/ 
		var $gnms_options;
		
		/**
		* @var array
		*/
		var $gnms_error_msgs;
		
		/**
		* Class constructor
		*/
		function __construct() {
			global $wpdb;
						
			/* initiate variables */
			$this->gnms_db = $wpdb->prefix . "gnms";
			$this->gnms_default_jpeg_quality = 88;
			$this->gnm_thumbnail_width = 46;
			$this->gnm_thumbnail_height = 60;
			$this->gnm_library = 'library';
			
			$this->gnms_error_msgs = array( "smallSourceWarningMsg" =>
				__("The source image is smaller than thumbnail's preset width or height", "Gnms") . ".\n\n"
				.__("Therefore, the thumbnail generated from this image will contain enlarged pixels", "Gnms")  . ".\n\n"
				.__("No minimum crop size restrain will be set","Wppt") . "." );

			/* Language localization */
			load_plugin_textdomain('Gnms',
				'wp-content/plugins/gm-media-selector/languages', 'gm-media-selector/languages');

			register_activation_hook( __FILE__, array(&$this, 'install') );

			add_action('init', array(&$this, 'init') );
			
			/* add AJAX actions */
			add_action( 'wp_ajax_getMediaSelectorFields', array(&$this, 'getMediaSelectorFields') );
			add_action( 'wp_ajax_getSrcAttachment', array(&$this, 'getSrcAttachment') );			
			add_action( 'wp_ajax_saveFileReference', array(&$this, 'saveFileReference') );
			add_action( 'wp_ajax_saveCropppedImage', array(&$this, 'saveCropppedImage') );
			add_action( 'wp_ajax_getFieldData', array(&$this, 'getFieldData') );
			add_action( 'wp_ajax_deleteMediaAsset', array(&$this, 'deleteMediaAsset') );
			add_action( 'wp_ajax_updateFieldOrder', array(&$this, 'updateFieldOrder') );
			
			if ( is_admin() ) {
				add_action( 'admin_print_styles', array(&$this, 'admin_header_css') );
				add_action( 'admin_print_scripts', array(&$this, 'admin_header_js') );
			}
			
			add_image_size( 'selector-thumbnail', $this->gnm_thumbnail_width, $this->gnm_thumbnail_height , TRUE); // Selector thumbnail size
		}
		
		/**
		* Add admin_header
		*/
		function admin_header_css(){
			wp_register_style( 'gnms_adminheader_css', WP_PLUGIN_URL . '/gn-media-selector/css/gnms.css' );
			wp_enqueue_style( 'gnms_adminheader_css');
			wp_register_style( 'gnms_jcrop_css', WP_PLUGIN_URL . '/gn-media-selector/css/jquery.Jcrop.css' );
			wp_enqueue_style( 'gnms_jcrop_css');
		}
		
		/**
		* Add admin_header
		*/
		function admin_header_js() {
			
			wp_register_script( 'gnms_jtools', WP_PLUGIN_URL . '/gn-media-selector/js/jquery.tools.min.js', array('jquery'), '0.9.8' );
			wp_enqueue_script( 'gnms_jtools' );
			wp_register_script( 'gnms_jcrop', WP_PLUGIN_URL . '/gn-media-selector/js/jquery.Jcrop.min.js', array('jquery'), '0.9.8' );
			wp_enqueue_script( 'gnms_jcrop' );
			wp_register_script( 'gnms_json2', WP_PLUGIN_URL . '/gn-media-selector/js/json2.min.js', array('jquery'));
			wp_enqueue_script( 'gnms_json2' );
			wp_register_script( 'gnms_crop', WP_PLUGIN_URL . '/gn-media-selector/js/gnmsCrop.js', array('jquery'));
			wp_enqueue_script( 'gnms_crop' );
			wp_register_script( 'gnms', WP_PLUGIN_URL . '/gn-media-selector/js/gnms.js', array('jquery'));
			wp_enqueue_script( 'gnms' );
		}
		
		/**
		* Returns default upload path where thumbnails are stored:
		* '/public_html/myblog/wp-content/uploads/gn-media-selector'
		* is the default directory if you have not changed it.
		*
		* @return array
		*/
		function getUploadPath() {
			return ABSPATH . str_replace( ABSPATH, '', trim( get_option( 'upload_path' ) ) ) .''.$this->gnm_library;
		}

		/**
		* Convert a thumbnail's URL to absolute file path
		*
		* @return string
		*/
		function urlToPath( $url ) {
			return $this->getUploadPath() . '/' . basename( $url );
		}

		/**
		* Convert a file's absolute path to URL location
		*
		* @return string
		*/
		function pathToUrl( $path ) {
			return get_option('siteurl') . '/' . trim( str_replace( ABSPATH, '', $path ), '/' );
		}
		
		/**
		* Create 'gn-media-selector' directory within default upload directory
		* if not found.
		*
		* Create plugin's database tables.
		*/
		function install() {
			/* Get this plugin's default upload directory. Default is */
			$upload_dir = $this->getUploadPath();

			/* Make '/wp-content/uploads/wp-post-thumbnail/' if not found */
			if ( !file_exists( $upload_dir ) ) {
				if ( wp_mkdir_p( $upload_dir ) ) { chmod( $upload_dir, 0777 ); }
				else { wp_die( sprintf ( __("Cannot create '%s' directory","Wppt"), $upload_dir ) ); }
			}

			if ( file_exists( $upload_dir ) ) { $this->createDB(); }
		}
		
		
		/**
		* Initialise plugin. Read or create options if not found.
		*/
		function init() {
			global $wpdb;
			
			/* Load gnms_options. Generate default values if not found */
			$this->gnms_options = get_option( 'gnms_options' );
			$this->display_errors = TRUE;
			
			if ( empty( $this->gnms_options ) ) {
				$this->gnms_options = array();
				$this->gnms_options['default_jpeg_quality'] = 88;
				$this->gnms_options['default_thumbnail'] = -1;
				$this->gnms_options['groups'] = array();
				update_option( 'gnms_options', $this->gnms_options, 'Global Native Media Selector options' );
			}
		}
		
		/**
		* Create 'gn-media-selector' database tables if not exists.
		* Run when plugin is activated
		*/
		function createDB() {
			global $wpdb;
			
			$create_gnms_table_sql = "CREATE TABLE IF NOT EXISTS $this->gnms_db ( " .
					"`gnms_id` bigint(20) NOT NULL auto_increment, " .
					"`gnms_group` text, " .
					"`gnms_post_id` bigint(20) NOT NULL default '0', " .
					"`gnms_field_index` bigint(20) default '-1', " .
					"`gnms_field_name` text, " .
					"`gnms_order` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_image` enum('y','n') NOT NULL default 'n', " .
					"`gnms_guid` text NOT NULL, " .
					"`gnms_thumb` text NOT NULL, " .
					"`gnms_src_id` bigint(20) default '0', " .
					"`gnms_x` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_y` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_x2` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_y2` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_orig_x` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_orig_y` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_orig_x2` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_orig_y2` SMALLINT UNSIGNED NOT NULL default '0', " .
					"`gnms_alt` text NOT NULL, " .
					"`gnms_title` text NOT NULL, " .
					"`gnms_caption` text NOT NULL, " .
					"`gnms_description` text NOT NULL, " .
					"PRIMARY KEY (`gnms_id`) " . ");";
			
			require_once( ABSPATH . 'wp-admin/upgrade-functions.php' );
			$wpdb->query("DROP TABLE IF EXISTS $this->gnms_db");
			dbDelta( $create_gnms_table_sql );
		}
		
		/**
		* Insert/Update thumbnail data in "prefix_gnms" database
		*
		* @param array
		* @return boolean
		*/	
				
		function updateThumbnailDB( $args ) {
			
			extract( $args );
			global $wpdb;
			//
			if ( $this->getRecordRow( $gnms_id ) != NULL ) {	
				$str = $wpdb->prepare(
					"UPDATE $this->gnms_db SET gnms_group = %s, gnms_field_index = %d, gnms_field_name = %s, gnms_image = %s, gnms_guid = %s, gnms_thumb =%s, gnms_src_id = %d, 
						gnms_x = %d, gnms_y = %d, gnms_x2 = %d, gnms_y2 = %d,
						gnms_orig_x = %d, gnms_orig_y = %d, gnms_orig_x2 = %d, gnms_orig_y2 = %d,
						gnms_alt = %s, gnms_title = %s, gnms_caption = %s, gnms_description = %s
					WHERE gnms_id = %d",
					"$gnms_group", $gnms_field_index, "$gnms_field_name", "$gnms_image", "$gnms_guid", "$gnms_thumb", intval( $gnms_src_id ),
					intval( $gnms_x ), intval( $gnms_y ), intval( $gnms_x2 ), intval( $gnms_y2 ),
					intval( $gnms_x ), intval( $gnms_y ), intval( $gnms_x2 ), intval( $gnms_y2 ),
					"$gnms_alt", "$gnms_title", "$gnms_caption", "$gnms_description",
					intval( $gnms_id ) );
				$result = $wpdb->query ( $str );
				//
				return ($result) ? $gnms_id : FALSE;
			} else {				
				$result = $wpdb->insert( $this->gnms_db, 
					array( 
						'gnms_group'=>"$gnms_group",
						'gnms_post_id'=>intval($gnms_post_id), 
						'gnms_field_index'=>intval($gnms_field_index),
						'gnms_field_name'=>"$gnms_field_name",
						"gnms_image"=>"$gnms_image",
						'gnms_order'=>intval($gnms_order), 
						'gnms_guid'=>"$gnms_guid", 
						'gnms_thumb' => "$gnms_thumb",
						'gnms_src_id'=>intval($gnms_src_id),
						'gnms_x'=>intval($gnms_x),
						'gnms_y'=>intval($gnms_y), 
						'gnms_x2'=>intval($gnms_x2),
						'gnms_y2'=>intval($gnms_y2), 
						'gnms_orig_x'=>intval($gnms_x),
						'gnms_orig_y'=>intval($gnms_y), 
						'gnms_orig_x2'=>intval($gnms_x2), 
						'gnms_orig_y2'=>intval($gnms_y2),
						'gnms_alt'=>"$gnms_alt", 
						'gnms_title'=>"$gnms_title",
						'gnms_caption'=>"$gnms_caption",
						'gnms_description'=>"$gnms_description"
					), 
					array( '%s', '%d', '%d', '%s', '%s', '%d', '%s', '%s', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%s', '%s', '%s', '%s' ) );				
				return ($result) ? $wpdb->insert_id : FALSE;
			}
		}
		
		/**
		* 
		*/
		function getFileTypes( $groupName , $groupFieldIndex = NULL )
		{
			$group = $this->getGroup($groupName);
			if(is_null($groupFieldIndex)) return isset($group['file_types']) ? $group['file_types'] : NULL ;
			return isset($group['fields'][$groupFieldIndex]['file_types']) ? $group['fields'][$groupFieldIndex]['file_types'] : NULL ;
		}
		
		/**
		* 
		*/
		function getMimeTypes( $groupName , $groupFieldIndex = NULL )
		{
			$group = $this->getGroup($groupName);
			if(is_null($groupFieldIndex)) return isset($group['mimes']) ? $group['mimes'] : NULL ;
			return isset($group[$fields][$groupFieldIndex]['mimes']) ? $group[$fields][$groupFieldIndex]['mimes'] : NULL ;
		}
		/**
		* 
		*/
		function getMime( $file_ext ){	
			$mimes = get_allowed_mime_types();
			
			foreach ( $mimes as $ext_preg => $mime_match ) {
				$ext_preg = '!(' . $ext_preg . ')$!i';				
				if ( preg_match( $ext_preg, $file_ext, $ext_matches ) ) {
					$type = $mime_match;
					$ext = $ext_matches[1];
					return $mime_match;
				}
			}
			return FALSE;	
		}
		
		/**
		* 
		*/
		function convertFileTypeToMime( &$options ){
			$fileTypesArray = array();
			
			if(!is_array($options['file_types'])) return;
			
			foreach( $options['file_types'] as $file_type )
			{
				$mime = $this->getMime( $file_type );
				// place n filetypes array is mime and not already there
				if( $mime && !in_array( $mime, $fileTypesArray ) ) $fileTypesArray[] = $mime;
			}
			
			$options['mimes'] = $fileTypesArray;
		}
		
		/**
		* 
		*/
		function convertMimeTypes( &$options ){
			
			// set group file type if needed
			if(is_array($options['file_types']))
				$this->convertFileTypeToMime( $options );
			
			// loop through fields if relevant and find file types
			if(is_array($options['fields']) && count($options['fields']) > 0)
				foreach( $options['fields'] as $key=>$field )
					$this->convertFileTypeToMime( $options['fields'][$key]); 
		}
		
		/**
		*
		*/
		function updateFieldOrder( ){
			global $wpdb;
			//
			$fieldsOrder = $_POST['order'];
			//
			foreach( $fieldsOrder as $field )
			{
				list($index, $id) = $field;				
				$wpdb->query ($wpdb->prepare("UPDATE $this->gnms_db SET gnms_order =%d WHERE gnms_id = %d", intval($index), intval($id)));
			}
			exit;
		}
		
		/**
		* Retrieve a row of record from "gnms" database
		* where "gnms_post_id" and "gnms_field_id" matches
		*
		* @return array or null
		*/
		function getRecordRow( $ID ){
			global $wpdb;
			return $wpdb->get_row($wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_id = %d", intval( $ID ) ));
		}
		
		/**
		* Get selected thumbnail's cropped coordinates.
		*
		* @return array
		*/
		function extractCropCoords( $results ) {
			return array( 'x' => intval( $results->gnms_x ), 'y' => intval( $results->gnms_y ),
					'w' => intval( $results->gnms_x2 ) - intval( $results->gnms_x ),
					'h' => intval( $results->gnms_y2 ) - intval( $results->gnms_y ) );
		}
		
		/**
		* Generate a random string of numbers and alphabets.
		* 
		* @return string
		*/
		function makeRandomString( $len = 12 ) {
			$chars = "PQO0WI1E2URY3TLA8KS7JD6HF2GVBCNXMZqazwsxedrf4cvty8bngh1uij0kmlpo";
			for ( $p = 0; $p < $len; $p++ ) { $s .= $chars[ mt_rand( 0, strlen( $chars ) ) ]; }
			return $s;
		}
		
		/**
		* Make filename out of keyword(s) entered by user.
		* Append a random string and optional $ext at the end.
		* Prepend $dir (upload path) at the start.
		*
		* Turns "nice thumbnail" keywords
		*    to "/media_libray/nice-thumbnail-rj8E2hDb" 
		*
		* @return string
		*/
		function makeThumbnailFilename( $name, $dir = '', $ext = '' ){
			
			if ( $dir == '' ) { $dir = $this->getUploadPath() . '/'; }
			if ( $ext != '' ) { $ext = '.' . $ext; }

			$filename = strtolower( str_replace( ' ', '-', $name ) );

			$hasExt = strrpos( $filename, '.' );
			if ( $hasExt ) { $filename = substr( $filename, 0, $hasExt ); }

			if ( strlen( $filename ) > 0 ) { $filename .= '-'; }
			$filename .= $this->makeRandomString( 6 );

			return "$dir$filename$ext";
		}
		
		/**
		* 
		*/
		function addMediaGroup( $groupName, $args, $override = FALSE ){
			global $post;
			//
			$defaults = array(
				'name'=>$groupName,
				'title'=>FALSE,
				'description'=>FALSE,
				'post_type'=>$post->post_type,
				'fields'=>FALSE,
				'max'=>-1,
				'sortable'=>FALSE,
				'width'=>-1,
				'height'=>-1,
				'jpeg_quality'=>-1,
				'file_types'=>FALSE,
				'multiple'=>FALSE
			);
		
			// cheekily use wp own parse args... thanks wp
			$options = wp_parse_args( $args, $defaults );
			
			// this is a pretty slow way to do this.. may have to fnd a nother way.
			$this->convertMimeTypes( $options );
			
			// if set fields then we dont want to be able to add 
			$options['multiple'] = ($options['max'] > 1 || $options['max'] == -1) ? TRUE : FALSE;
			if(is_array($options['fields']) && count($options['fields']) > 0) $options['multiple'] = FALSE;
				
			// new group so add
			if(!$this->gnms_options['groups'][$groupName]){
				error_log('adding group '.$groupName);
				$this->gnms_options['groups'][$groupName] = $options;
			// check if groupName is already taken and if the group options have changed
			}else if($this->gnms_options['groups'][$groupName] && !$override && $this->gnms_options['groups'][$groupName] != $options){
				// throw an warning that the admin will ned to override if they want to change the gorup options..
				if($this->display_errors)
					echo '<div class="error"><p>' . __("The supplied name \"$groupName\" is already taken. If you wish to overwrite this please set override to TRUE. This will remove all assets associated to this group that dont match the new conditions.",'gnms') . '</p></div>';
				return;
			// alright change the options..
			}else if($this->gnms_options['groups'][$groupName] && $this->gnms_options['groups'][$groupName] != $options && $override ){
				if($this->display_errors)
					echo '<div class="error"><p>' . __("Overriding group \"$groupName\". This is removing all assets associated to this group that don't match the new conditions.",'gnms') . '</p></div>';
				// cleaout all assets and DB Entries tha dont match our new group..
				$this->gnms_options['groups'][$groupName] = $options;
			}
			
			// obvious
			update_option( 'gnms_options', $this->gnms_options, 'Global Native Media Selector options' );
		}
		
		/**
		* 
		*/
		function removeMediaGroup($groupName)
		{
			error_log('removing group '.$groupName);
			//
			unset($this->gnms_options['groups'][$groupName]);
			//
			update_option( 'gnms_options', $this->gnms_options, 'Global Native Media Selector options' );
			//
			$this->flushMediaAssets();
		}
		
		/**
		* checks for orphaned media groups and removes DB entries and assets
		*/
		function flushMediaAssets(){
			global $wpdb;
			
			error_log('flushMediaAssets');
			
			$groups = $this->gnms_options['groups'];
			
			// ----------------------------------------------------------------------
			// check groups
			// ----------------------------------------------------------------------
			$unique_groups = $wpdb->get_results("SELECT distinct gnms_group FROM $this->gnms_db" );
			
			foreach($unique_groups as $key=>$value)
			{
				error_log('flushMediaAssets :: searching - '.$value->gnms_group);
				// check for entry in options
				if(!isset($this->gnms_options['groups'][$value->gnms_group]))
				{
					error_log('flushMediaAssets :: found a group that doesn\'t belong in DB');
					// get all entries in DB...
					$groupAssets = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_group = '%s'", $value->gnms_group ) );
					//
					foreach($groupAssets as $asset)
					{
						error_log('flushMediaAssets :: Attempting to delete asset '.$asset->gnms_id);
						$deletedThumbID = $this->deleteThumbnail( $asset->gnms_id );
						error_log('flushMediaAssets :: Deleted asset '.$deletedThumbID);
					}
					// remove group DB					
					$wpdb->query( $wpdb->prepare( "DELETE FROM $this->gnms_db WHERE gnms_group = '%s'", $value->gnms_group ) );	
				}
			}
			
			// ----------------------------------------------------------------------
			// check assets
			// ----------------------------------------------------------------------
			
			
		}
		
		/**
		* 
		*/
		function getMediaSelector($groupName)
		{
			global $post;	
								
			$group = $this->gnms_options['groups'][$groupName];
			//
			return gnms_selector($groupName, $post->ID, $group, $this->getFields( $groupName, $post->ID ));
		}
		
		/**
		* 
		*/
		function getMediaSelectorFields( $groupName = FALSE, $postID = FALSE ){
			global $post;
			
			if($groupName == FALSE && isset($_POST['groupName'])) $groupName = $_POST['groupName'];
			if($postID == FALSE && isset($_POST['postID'])) $postID = $_POST['postID'];			
			if($postID == FALSE && isset($post->ID)) $postID = $post->ID;
			if(!$postID || !$groupName) exit;			
			echo json_encode( $this->getGroup( $groupName, $postID ,TRUE ) );
			exit;
		}
		
		/**
		* 
		*/
		function getGroup( $groupName, $postID = NULL, $includeFieldAssets = FALSE ){
			$group = $this->gnms_options['groups'][$groupName];

			if(!is_null($postID) && $includeFieldAssets)
			{
				$group['fields'] = $this->getFields( $groupName, $postID );
			}

			return $group;
		}
		
		/**
		* 
		*/
		function sanitizeFieldData( $asset )
		{
			//
			$field = array();
			$field['ID'] = $asset->gnms_id;
			$field['order'] = $asset->gnms_order;
			$field['srcID'] = $asset->gnms_src_id;
			$field['groupName'] = $asset->gnms_group;
			$field['groupFieldIndex'] = $asset->gnms_field_index;
			$field['postID'] = $asset->gnms_post_id;
			$field['data'] = array(
								'title' => $asset->gnms_title,
								'alt' => $asset->gnms_alt,
								'caption' => $asset->gnms_caption,
								'description' => $asset->gnms_description
								);
			// do we have an image to work with
			if($asset->gnms_image == 'y')
			{
				$field['thumbnail'] = array(
									'src' => $asset->gnms_thumb,
									'width' => $this->gnm_thumbnail_width,
									'height' => $this->gnm_thumbnail_height
									);

				$field['image'] = array(
									'src' => $asset->gnms_guid,
									'title' => $asset->gnms_title,
									'alt' => $asset->gnms_alt
									);
			}else{

				list( $src, $width, $height) = wp_get_attachment_image_src( $asset->gnms_src_id, 'selector-thumbnail', true );

				$field['thumbnail'] = array(
									'src' => $src,
									'width' => $width,
									'height' => $height
									);

			}					
			return $field;
		}
		/**
		* 
		*/
		function getFields( $groupName, $postID ){
			global $wpdb;
							
			$group = $this->gnms_options['groups'][$groupName];
			$assets = $wpdb->get_results("SELECT * FROM $this->gnms_db WHERE gnms_post_id = $postID AND gnms_group = '$groupName' ORDER BY gnms_order" );
			
			if(is_array($group['fields']) && count($group['fields']) > 0)
			{
				$fields = $group['fields'];
				foreach($fields as $key=>$field)
				{
					$fields[$key]['groupFieldIndex'] = $key;
				}
				
				foreach($assets as $asset)
				{
					if( !is_null($asset->gnms_field_index) 	)
					{
						// lets use wp parse args function to include the original field data						
						$fields[$asset->gnms_field_index] = wp_parse_args( $this->sanitizeFieldData($asset), $fields[$asset->gnms_field_index] );
					}
				}
			}else{
				if($assets)
				{
					$fields = array();
					// since we dont ave any pre defined field data we'll 
					// populate the fields straight from the database
					foreach($assets as $asset)
						$fields[] = $this->sanitizeFieldData($asset);
					
				}else{
					//echo "NO FIELDS";
				}
			}
						
			return $fields;
		}

		
		/**
		* 
		*/
		function getMediaAssets( $postID, $groupName, $nameOrFieldIndex = NULL )
		{
			global $wpdb;
			
			// if user is asking for a singleitem
			if(!is_null($nameOrFieldIndex))
			{
				//
				$query = (is_int($nameOrFieldIndex)) ?
					$wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_group = %s AND gnms_post_id = %d AND gnms_field_index = %d", $groupName , $postID, $nameOrFieldIndex ):
					$wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_group = %s AND gnms_post_id = %d AND gnms_field_name = %s", $groupName , $postID, $nameOrFieldIndex );
				
				$asset = $wpdb->get_row( $query );
				
				return  ($asset) ? $this->sanitizeFieldData($asset) : FALSE;	
			}else{
				$assets = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_group = %s AND gnms_post_id = %d", $groupName , $postID ) );
				
				if(!$assets || count($assets) == 0) return FALSE;
				//
				$fields = array();
				foreach($assets as $asset)
				{
					$fields[] =  $this->sanitizeFieldData($asset);
				}
				return $fields;
			}	


		}
		
		/**
		* 
		*/
		function getAttachments( $groupName = NULL, $groupFieldIndex = NULL, $use_post = FALSE ){
			
			global $wpdb;
			//
			$assets_args = array();
			//
			if(!is_null($groupName))
			{
				$group = $this->getGroup($groupName);
				
				if(!is_null($groupFieldIndex)&& isset($group['fields'][$groupFieldIndex]) && isset($group['fields'][$groupFieldIndex]['mimes']))
							$mimes = $group['fields'][$groupFieldIndex]['mimes'];
			
				// default to group file_types if none set for fields
				if(!$mimes && !is_array($mimes) && $group['mimes']) $mimes = $group['mimes'];		
			}		

			if(!empty($_GET['postID']) && $use_post )
				$assets_args['post_parent'] = $_GET['postID'];
			
			//
			$argsStr = '';
			foreach($assets_args as $key=>$value)
			{
				$argsStr .= " AND $key = $value";
			}
			
			$queryStr = "SELECT * FROM $wpdb->posts WHERE post_type = 'attachment' ".$argsStr;
			
			if(is_array($mimes))
			{
			 	$queryStr .= " AND post_mime_type IN ( '".join("' , '", $mimes )."' )";
			}
			
			return $wpdb->get_results( $queryStr );
		}

		/**
		* Called via AJAX whenever a thumbnail tab is clicked.
		*
		* @return JSON
		*/
		function getFieldData( $ID = NULL, $echoJSON = TRUE ) {
		
			//get current row
			$asset = $this->getRecordRow(  (  (is_null($ID) || !isset($ID) || $ID == '' ) ? $_POST['ID'] : $ID)  );
			//
			$field = array();
			$field['ID'] = $asset->gnms_id;
			$field['order'] = $asset->gnms_order;
			$field['srcID'] = $asset->gnms_src_id;
			$field['groupName'] = $asset->gnms_group;
			$field['groupFieldIndex'] = $asset->gnms_field_index;
			$field['postID'] = $asset->gnms_post_id;
			$field['data'] = array(
								'title' => $asset->gnms_title,
								'alt' => $asset->gnms_alt,
								'caption' => $asset->gnms_caption,
								'description' => $asset->gnms_description
								);
															
			if($asset->gnms_image == 'y')
			{
				$field['thumbnail'] = array(
									'src' => $asset->gnms_thumb,
									'width' => $this->gnm_thumbnail_width,
									'height' => $this->gnm_thumbnail_height
									);
									
				$field['image'] = array(
									'src' => $asset->gnms_guid,
									'title' => $asset->gnms_title,
									'alt' => $asset->gnms_alt
									);
			}else{
				
				list( $src, $width, $height) = wp_get_attachment_image_src( $asset->gnms_src_id, 'selector-thumbnail', true );
				
				$field['thumbnail'] = array(
									'src' => $src,
									'width' => $width,
									'height' => $height
									);
				
			}

			// add any extra data
			if($asset->gnms_field_index != -1)
			{				
				$group = $this->gnms_options['groups'][$asset->gnms_group];
				$fields = $group['fields'];
				
				$ofield = wp_parse_args( $field, $fields[$asset->gnms_field_index] );
				
				if($echoJSON)
				{
					echo json_encode( $ofield );
					exit();
				}else{
					return $ofield;
				}
							
			}
			
			if($echoJSON)
			{
				echo json_encode( $field );
				exit();
			}else{
				return $field;
			}
		}
		
		/**
		* Get source image attachment
		*/
		function getSrcAttachment() {
			global $wpdb;
			
			$group = $this->getGroup( attribute_escape( $_POST['groupName'] ));
			
			//
			if(isset($_POST['groupFieldIndex']) && $_POST['groupFieldIndex'] != intval(-1))
			{
				$field = $group['fields'][ $_POST['groupFieldIndex'] ];
			}

			
			// 
			$data['srcID'] = attribute_escape( $_POST['srcID'] );
			$data['warning_smallSrcImg'] = $this->gnms_error_msgs['smallSourceWarningMsg'];

			/* get absolute path source attachment image */
			$data['abspath'] = get_attached_file( $data['srcID'] );
			$asset_meta 	= wp_get_attachment_metadata( $data['srcID'] );
			$asset 			= get_post( $_POST['srcID'] );
			
			//
			if( preg_match('!^image/!', get_post_mime_type( $asset )) && file_is_displayable_image($asset->guid) )
			{
				
				$containerWidth = intval( attribute_escape( $_POST['containerWidth'] ) );
				$containerHeight = intval( attribute_escape( $_POST['containerHeight'] ) );
				
				/* Get attachment image URL, full width, full height */
				$attachImgData = wp_get_attachment_image_src( $data['srcID'], 'full' );			

				if ( !$attachImgData || !file_exists($data['abspath'] ) ) {
					$data['error'] =
						sprintf( __("[ Error ] : Image attachment: \n\n \" %s \" \n\n ... is not found.", "Wppt"), $data['srcID'] );
					echo json_encode( $data );
					exit;
				}
			
				$cache_dir = WP_PLUGIN_DIR . '/gn-media-selector/cache';
				if ( !file_exists( $cache_dir ) ) {
					$data['error'] =
						sprintf( __("[ Error ] : WP Post Thumbnail's cache directory: \n\n \" %s \" \n\n ... does not exists.", "Wppt"), $cache_dir );
					echo json_encode( $data );
					exit;
				};
			
				if ( !is_writable( $cache_dir ) ) {
					$data['error'] =
						sprintf( __("[ Error ] : WP Post Thumbnail's cache directory: \n\n \" %s \" \n\n ... is not writable. \n\n [ Fix ] : CHMOD this directory to 777 mode.", "Wppt"), $cache_dir );
					echo json_encode( $data );
					exit;
				};
			
				$data = array_merge( $data, $attachImgData );
			
				/* resize image attachment to fit cropping area if necessary */
				$heightScale = $containerHeight / $data[2];
				$widthScale = $containerWidth / $data[1];
				$data['scaleRatio'] = $widthScale <= $heightScale ? $widthScale : $heightScale;
				$data['scaleRatio'] = min( $data['scaleRatio'], 0.99 );
			
				$cache_dir = WP_PLUGIN_DIR . '/gn-media-selector/cache';
				array_map( 'unlink', glob( $cache_dir . '/*.*' ) );
				$data['imageSrc'] = $this->pathToUrl( image_resize( $data['abspath'],
					$data[1] * $data['scaleRatio'], $data[2] * $data['scaleRatio'], false, "", $cache_dir, 90 ) );
				
				$data['imageWidth'] = ($data[1] * $data['scaleRatio']);
				$data['imageHeight'] = ($data[2] * $data['scaleRatio']);
			
				//
				$data['thumbWidth'] = ( isset($field) && isset($field['width']) ) ? 
												$field['width'] : 
														(( $group['width'] != -1 ) ? $group['width'] : NULL);
				//									
				$data['thumbHeight'] = ( isset($field)  && isset($field['height']) ) ? 
												$field['height'] : 
														(( $group['width'] != -1 ) ? $group['height'] : NULL);	
														
				$data['isImage'] = TRUE;
			}else{
				$data['isImage'] = FALSE;
				list( $src, $width, $height) = wp_get_attachment_image_src( $asset->ID, 'selector-thumbnail', true );
				$data['imageSrc'] = $src;
				$data['imageWidth'] = $width;
				$data['imageHeight'] = $height;
				
			}
			
			/* If this is source image for selected thumbnail, get crop coordinates from database */			
			if(isset( $_POST['ID'] ) )
			{
				$row = $this->getRecordRow( intval($_POST['ID']) );
				
				//
				if ( $row->gnms_src_id == $data[ 'srcID' ] )
						$data = array_merge( $data, $this->extractCropCoords( $row ) );

				$data['title'] = $row->gnms_title;
				$data['alt_title'] = $row->gnms_alt;
				$data['caption'] = $row->gnms_caption;
				$data['description'] = $row->gnms_description;
			}

						
			if(!isset($data['title']) || $data['title'] == '') $data['title'] = $asset->post_title;
			if(!isset($data['alt_title']) || $data['alt_title'] == '') $data['alt_title'] = $asset->alt_title;
			if(!isset($data['caption']) || $data['caption'] == '') $data['caption'] = $asset->caption;
			if(!isset($data['description']) || $data['description'] == '') $data['description'] = $asset->caption;
			
						
			echo json_encode( $data );
			exit;
		}
		
		
		function saveFileReference() {
			//
			$asset = get_post( $_POST['srcID'] );
			
			$group = $this->getGroup( attribute_escape($_POST['groupName']));
			$field = $group['fields'][ attribute_escape( $_POST['groupFieldIndex'] )];
			// /* Update "gnms" database table with thumbnail's info */
			$result = $this->updateThumbnailDB( array(
					'gnms_id'=>( isset($_POST['ID']) ? intval($_POST['ID']) : NULL ),
					'gnms_group'=> $_POST['groupName'],
					'gnms_post_id' => $_POST['postID'],
					'gnms_field_index' => $_POST['groupFieldIndex'],
					'gnms_field_name' => (isset($field) && $field['name']) ? $field['name'] : '',
					'gnms_src_id' => $_POST['srcID'],
					'gnms_title' => $_POST['title'],
					'gnms_alt' => $_POST['altTitle'],
					'gnms_caption'=>$_POST['caption'],
					'gnms_description'=>$_POST['description']
					 ) );
			//
			$output = ($result) ? $this->getFieldData( $result, FALSE ) : FALSE;
			echo json_encode( $output );
			exit;
			
		}
		/**
		* Save cropped image as thumbnail
		*
		* @return string
		*/
		function saveCropppedImage()
		{
			global $wpdb;	
			
			error_log( "" );
			error_log( "-------------------------------------------------" );
			error_log( "saveCropppedImage" );
			error_log( "-------------------------------------------------" );
					
			//
			$group = $this->getGroup( attribute_escape( $_POST['groupName'] ));
			$field = $group['fields'][ attribute_escape( $_POST['groupFieldIndex'] )];
			
			//
			$asset = get_post( $_POST['srcID'] );
			$asset_meta = wp_get_attachment_metadata($asset->ID);
			
			/* cropped thumbnail dimensions */
			$x = intval( $_POST['x'] );     $y = intval( $_POST['y'] );
			$x2 = intval( $_POST['x2'] );   $y2 = intval( $_POST['y2'] );
			$w = intval( $_POST['w'] ); $h = intval( $_POST['h'] );
			
			error_log( "Coords ~ ".$x." : ".$y." -- ".$x2." : ".$y2);
			error_log( "Size ~ ".$w." : ".$h);
			
			$jpeg_quality = ( isset($field['jpeg_quality']) && $field['jpeg_quality'] != -1 ) ? 
													$field['jpeg_quality'] : 
															( $group['jpeg_quality'] != -1 ) ? $group['jpeg_quality'] : $this->gnms_options['default_jpeg_quality'];

			$srcPath = get_attached_file( $asset->ID ); /* get source image path */
				
			$thumb_width = ( isset($field) && $field['width'] ) ? 
											$field['width'] : 
													(( $group['width'] != -1 ) ? $group['width'] : $w);
			$thumb_height = ( isset($field) && $field['height'] ) ? 
											$field['height'] : 
													(( $group['width'] != -1 ) ? $group['height'] : $h);
													
			/* Convert keywords "my cool thumbnail" to "my-cool-thumbnail-4ij2Pu".
			Does not have extension (.jpg, .png) yet. */
			$assetname = ( isset($_POST['title'])  && $_POST['title'] != '') ? $this->makeThumbnailFilename( attribute_escape( $_POST['title'] ) ) : $this->makeThumbnailFilename( $asset->post_title );
			
			/* Generate and save new thumbnail. Returns "my-cool-thumbnail-4ij2Pu.jpg"
			if thumbnail is a .jpg file. */
			require_once ('includes/gd_image_processing.php');
			$thumb_absolute = gdResizeThumbnail( $assetname, $srcPath, $x, $y, $x2 - $x, $y2 - $y, $thumb_width, $thumb_height, $jpeg_quality );
			$thumb_guid = $this->pathToUrl( $thumb_absolute );
			
			// resized thumb
			$thumb = image_make_intermediate_size( $thumb_absolute, $this->gnm_thumbnail_width, $this->gnm_thumbnail_height, TRUE );
			$thumbSrc = $this->pathToUrl( $this->getUploadPath().'/'.$thumb['file'] );
			
			//  error_log( "~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~" );		
			//  error_log( 'ID - '.intval($_POST['ID']) );
			//  error_log( 'groupName - '.$_POST['groupName']);
			//  error_log( 'postID - '.intval($_POST['postID']) );								
			//  error_log( 'gnms_field_index - '.intval($_POST['gnms_field_index']) );								
			//  error_log( 'gnms_src_id - '.intval($_POST['gnms_src_id']) );								
			//  error_log( '' );													
			//  error_log( 'assetname - '.$assetname );
			//  error_log( 'srcPath - '.$srcPath );
			// error_log( 'coords - '.$x." : ".$y.' -- '.$x2." : ".$y2 );
			// error_log( 'thumb - '.$thumb_width." : ".$thumb_height );
			// error_log( 'jpeg_quality - '.$jpeg_quality );
			// error_log( '$thumb_guid - '.$thumb_guid );
			// error_log( "~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~" );		
			
			
			
			/* Check for and remove old thumbnail if exists */
			$this->deleteThumbnail( intval($_POST['ID']) );
			
			// /* Update "gnms" database table with thumbnail's info */
			$result = $this->updateThumbnailDB( array(
					'gnms_id'=>( isset($_POST['ID']) ? intval($_POST['ID']) : NULL ),
					'gnms_group'=> $_POST['groupName'],
					'gnms_post_id' => $_POST['postID'],
					'gnms_field_index' => $_POST['groupFieldIndex'],
					'gnms_field_name' => (isset($field) && $field['name']) ? $field['name'] : '',
					'gnms_image' => TRUE,
					'gnms_guid' => $thumb_guid,
					'gnms_src_id' => $_POST['srcID'],
					'gnms_thumb' => $thumbSrc,
					'gnms_x' => $x, 'gnms_y' => $y,
					'gnms_x2' => $x2, 'gnms_y2' => $y2,
					'gnms_alt' => $_POST['altTitle'],
					'gnms_title' => $_POST['title'],
					'gnms_caption'=>$_POST['caption'],
					'gnms_description'=>$_POST['description']
					 ) );
			
			//
			$output = ($result) ? $this->getFieldData( $result, FALSE ) : FALSE;
			echo json_encode( $output );
			exit;
		} /* save_thumbnail */
		
		/**
		* Delete thumbnail if exists
		*
		* @return int when successful or boolean if fail
		*/
		function deleteThumbnail( $ID ) {
			
			$row = $this->getRecordRow( $ID );
			
			//
			if ( !$row ) { return false; }
			
			$thumb_filepath = $this->urlToPath( $row->gnms_thumb );
			
			if ( file_exists( $thumb_filepath ) ) { 
				unlink( $thumb_filepath ); 
			}
			
			//
			$image_filepath = $this->urlToPath( $row->gnms_guid );
			
			/* Check if exist and delete it */
			if ( file_exists( $image_filepath ) ) { 
				unlink( $image_filepath ); 
			}
			else { return false; }

			return $row->gnms_id;
		}
		
		/**
		* Called via AJAX by user. Delete thumbnail and removes
		* record row from database.
		*/
		function deleteMediaAsset() {
			/* Delete the thumbnail file */
			$deletedThumbID = $this->deleteThumbnail( $_POST['ID'] );
			
			/* Remove thumbnail's record from database */
			if ( $deletedThumbID ) {
				global $wpdb;
				//
				$wpdb->query( $wpdb->prepare( "DELETE FROM $this->gnms_db WHERE gnms_id = %d", $deletedThumbID ) );				
				echo json_encode( TRUE );
			}else{
				echo json_encode( FALSE );
			}
			exit;
		}
		
		/**
		* Called via AJAX by user. Delete thumbnail and removes
		* record row from database.
		*/
		function removeMediaGroupAssets( $groupName = NULL, $postID = NULL)
		{
			global $wpdb;
			
			if(is_null($groupName) || is_null($postID)) return;

			$groupAssets = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $this->gnms_db WHERE gnms_group = '%s' AND gnms_post_id = %d", $groupName , $postID ) );
			
			foreach($groupAssets as $asset)
			{
				$this->deleteThumbnail( $asset->gnms_id );
				$wpdb->query( $wpdb->prepare( "DELETE FROM $this->gnms_db WHERE gnms_id = %d", $asset->gnms_id ) );				
				
			}
		}
		
	} /* class GNMS */

} /* if ( !class_exists( "GNMS" ) ) */


/* instantiate class */
//if (class_exists( "Gnms")) { 
//	$gnms = new Gnms( );
// }

if( !isset($gnms) && class_exists( "Gnms")  ) {
	// Create a new instance of your plugin that utilizes the WordpressPluginFramework and initialize the instance.
	$gnms = new Gnms();
}

//
function gnMediaSelectorAdd( $groupName, $args = array(), $override = FALSE)	
{
	global $gnms;	
	//	
	$gnms->addMediaGroup($groupName, $args, $override);
	$gnms->getMediaSelector($groupName);
}

function gnMediaSelectorRemoveAssets($groupName, $postID )
{
	global $gnms;
	$gnms->removeMediaGroupAssets($groupName, $postID);
}

function gnMediaSelectorRemove($groupName)
{
	global $gnms;
	$gnms->removeMediaGroup($groupName);
}
// 
function gnMediaSelectorGet(  $groupName, $postID, $nameOrIndex = NULL )
{
	global $gnms;
	return $gnms->getMediaAssets( $postID, $groupName, $nameOrIndex );
}



