/*
*   SoundCloud Custom Player jQuery Plugin
*   Author: Matas Petrikas, matas@soundcloud.com
*   Copyright (c) 2009  SoundCloud Ltd.
*   Licensed under the MIT license:
*   http://www.opensource.org/licenses/mit-license.php
*
*   Usage:
*   <a href="http://soundcloud.com/matas/hobnotropic" class="sc-player">My new dub track</a>
*   The link will be automatically replaced by the HTML based player
*/
; (function($) {

    var debug = true,
    useSandBox = false,
	uniqueFieldCount = 0,
	
    log = function(args) {
        if (debug && window.console && window.console.log) {
            window.console.log.apply(window.console, arguments);
        }
    },
	cacheBuster = function()
	{
		return '?'+new Date().getTime();
	}
	removeMediaElement = function( $element, $gnMediaSelector  )
	{		
		if( $element.data('fieldData').ID ){
			
			var $imageHolder = $element.find('.gnms-image-holder');
			var $assetInfoTitle = $element.find('.gnms-asset-info-title');
			var $assetInfoAltTitle = $element.find('.gnms-asset-info-alt-title');
			
			
			$data = {
	            action: 'deleteMediaAsset',
	            'cookie': encodeURIComponent(document.cookie),
				'ID':$element.data('fieldData').ID
	        }

	        $.ajax({
	            dataType: 'json',
	            data: $data,
	            success: function(data) {
					// clear element..
					if($element.data('gnMediaSelector').data('groupData').multiple)
						$element.remove();
					else
						$element.data('fieldData',null);
						
					//
					$assetInfoTitle.empty();
					$assetInfoAltTitle.empty();
					$imageHolder.empty()					
				
					checkMultipleSelection($gnMediaSelector);
	            },
	            error: function(XMLHttpRequest, textStatus, errorThrown) {
	                log("Error :: " + textStatus + " : " + errorThrown);
	            },
	            complete: function() {}
	        });
		};
	}
	createMediaElement = function( $gnMediaSelector, $fieldData )
	{		
		// make complete list element
		var li = $('<li></li>');
		//li.attr('id','gnms-media-element-'+ (($fieldData.ID) ? $fieldData.ID : fieldData.groupFieldIndex));
		li.data('fieldData',$fieldData);
		li.data('gnMediaSelector',$gnMediaSelector);
		
		if($gnMediaSelector.data('groupData').sortable)
		{
			li.css('cursor','move');
		}
			
		var $imageHolder = $('<div class="gnms-image-holder"></div>');
		// add image if there is one
		if($fieldData.thumbnail && $fieldData.thumbnail.src){			
			var img = new Image();
			$(img).attr({ src:$fieldData.thumbnail.src+cacheBuster(), width:$fieldData.thumbnail.width, height:$fieldData.thumbnail.height } ).load( function() { });
			$imageHolder.append(img);
		}
		
		var title = (($fieldData.title) ? $fieldData.title : ' ~ ');
		var description = (($fieldData.description) ? $fieldData.description : ' ~ ');
		
		var $assetInfoTitle = $('<h3 class="gnms-asset-title"></h3>').text( title );
		var $assetInfoDescription = $('<p class="gnms-asset-descripion"></p>').text( description );
				
		var imageTitle = (($fieldData.data && $fieldData.data.title) ? ('Title: '+$fieldData.data.title) : '');
		var imageAlt = (($fieldData.data && $fieldData.data.alt) ? ('Alt: '+$fieldData.data.alt) : '');
		//		
		var $assetInfoImageTitle = $('<p class="gnms-asset-info-title"></p>').text(imageTitle.truncate(25));
		var $assetInfoImageAlt = $('<p class="gnms-asset-info-alt-title"></p>').text(imageAlt.truncate(25));
		var $assetInfo = $('<div class="gnms-asset-info"></div>');
		
		//
		if($fieldData.title)
			$assetInfo.append($assetInfoTitle);
		if($fieldData.description)
			$assetInfo.append($assetInfoDescription);
		
		$assetInfo.append($assetInfoImageTitle).append($assetInfoImageAlt);
		
		//
		var $selectButton = $('<a class="gnms-select-media" title="Select an Image" style="text-decoration:none;"><input type="button" class="gnms-select-asset-button button" value="Select media"/></a>');
		$selectButton.click(function(event) { 
			//var liParent = $(this).parent().parent('li');
			li.data('gnMediaSelector').trigger({type:'selectElement.gnMediaSelector', element: li });
		 });
		
		//
		var $deleteButton = $('<a title="Delete asset" class="gnms-delete-media" style="text-decoration:none;margin-top:10px;"><input type="button" class="gnms-delete-asset-button button" value="Delete Asset" /></a>');
		$deleteButton.click(function(event) { 
			//var liParent = $(this).parent().parent().parent('li');
			//liParent.data('gnMediaSelector').trigger({type:'removeElement.gnMediaSelector', element: liParent});
			li.data('gnMediaSelector').trigger({type:'removeElement.gnMediaSelector', element: li});
			
		});
		
		var $assetButtons = $('<div class="gnms-asset-buttons"></div>').append($selectButton).append($deleteButton);


		//
		var $wrapper = $('<div class="gnms-asset-wrapper"></div>')
		$wrapper.append($imageHolder);
		$wrapper.append($assetInfo);
		$wrapper.append($assetButtons);
		
		//
		return li.append($wrapper).append('<div class="clear"></div>');
	},
	updateMediaElement = function( $element,  $groupName, $ID )
	{
		var $imageHolder = $element.find('.gnms-image-holder');
		var $assetInfoTitle = $element.find('.gnms-asset-info-title');
		var $assetInfoAltTitle = $element.find('.gnms-asset-info-alt-title');
				
        loadFieldData = function( $ID ) {
			
            $data = {
                action: 'getFieldData',
                'cookie': encodeURIComponent(document.cookie),
				'ID':$ID
            }

            $.ajax({
                dataType: 'json',
                data: $data,
                success: function(data) {
					$element.data('fieldData',data);
					// update ID
					//$element.attr('id','gnms-media-element-'+data.ID);
					
					var imageTitle = ((data.data && data.data.title) ? "Title :" + data.data.title : '');
					var imageAlt = ((data.data && data.data.alt) ? "Alt :" + data.data.alt : '');
					//
					$assetInfoTitle.text(imageTitle.truncate(25));
					$assetInfoAltTitle.text(imageAlt.truncate(25));
										
					if(data.thumbnail && data.thumbnail.src){
						var img = new Image();
						$(img).attr({ src:data.thumbnail.src+cacheBuster(), width:data.thumbnail.width, height:data.thumbnail.height } ).load( function() { });
						$imageHolder.empty().append(img);
					}
					
					updateMediaOrder($element.data('gnMediaSelector'))
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    log("Error :: " + textStatus + " : " + errorThrown);
                },
                complete: function() {}
            });
        };

        // load first tracks
        loadFieldData( $ID );
		
	},
    loadSelectorData = function($gnMediaSelector, $groupName, $postID) {
        var index = 0,
        selectorObj = {
            node: $gnMediaSelector,
            data: {}
        },
        loadGroupData = function($groupName , $postID) {

            $data = {
                action: 'getMediaSelectorFields',
                'cookie': encodeURIComponent(document.cookie),
                'groupName': $groupName,
                'postID': $postID
            }

            $.ajax({
                dataType: 'json',
                data: $data,
                success: function(data) {
                    // if loading finishes, anounce it to the GUI
                    selectorObj.node.trigger({type:'onDataLoaded.gnMediaSelector', groupData: data});

                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    log("Error :: " + textStatus + " : " + errorThrown);
                },
                complete: function() {}
            });
        };
        // load first tracks
        loadGroupData($groupName,$postID);
    },
	updateMediaOrder = function($gnMediaSelector)
	{		
		var fieldList = $gnMediaSelector.find('.gnms-list li');
		var orderList = [];
		
		$.each(fieldList, function(index, field) {
			var data = $(field).data('fieldData');
			if(data)
				orderList.push([index, data.ID]);
		});
		
		sendFieldOrder= function( $order ) {

            $.ajax({
                dataType: 'json',
                data: {
	                action: 'updateFieldOrder',
	                'cookie': encodeURIComponent(document.cookie),
					'order':$order
	            },
                success: function(data) {
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    log("Error :: " + textStatus + " : " + errorThrown);
                },
                complete: function() {}
            });
        };

		sendFieldOrder(orderList);
	},
	checkMultipleSelection = function( $gnMediaSelector )
	{
		//
		var $groupData = $gnMediaSelector.data('groupData');
		
		// set add button
		if(( ($('.gnms-list li',$gnMediaSelector).length < $groupData.max ) && $groupData.multiple == true) || ($groupData.max == -1 && $groupData.multiple == true)){
			$gnMediaSelector.find('.gnms-add-media-button a').removeAttr('disabled');
			return true;
		}else{
			$gnMediaSelector.find('.gnms-add-media-button a').attr('disabled',true);
			return false;
		}
	}

    // Generate custom skinnable HTML/CSS/JavaScript based SoundCloud players from links to SoundCloud resources
    $.gnMediaSelector = function(options, node) {
        var opts = $.extend({},$.fn.gnMediaSelector.defaults, options),
		
		//
        $gnMediaSelector = node && $(node);
		$gnMediaList = $gnMediaSelector.append('<ul class="gnms-list"></ul>');

		
		
		
		// set opts if needed
		if(!opts.postID)
			if(typeof postID != "undefined") opts.postID = postID; else return null;
		if(!opts.groupName)
        	opts.groupName = $gnMediaSelector.attr('id');
		
		// init the player GUI, when the tracks data was laoded
        $gnMediaSelector.bind('onDataLoaded.gnMediaSelector', function(event) {
			
			//event.groupData.multiple = false;
			var gnMediaSelector = $(this).data('groupData',event.groupData);
			var $gnMediaElement = $("#"+event.groupData.name);			
			var fields = event.groupData.fields;
			//
			if(fields)
			{
				// create fields
				$.each(fields, function(index, field) {
					$gnMediaSelector.find('ul').append( createMediaElement( gnMediaSelector, this) );
				});
			}
			
			//
			if(event.groupData.multiple == true)
			{
				var gnAddMedia = $('<div class="gnms-add-media-button"><a title="Add Asset" style="text-decoration:none;" class="button">Add Asset</a></div>');		
				// add media button
		 		gnAddMedia.find('a').data('gnMediaSelector',$gnMediaSelector).click(function(){ 
					if(!$(this).attr('disabled'))
						$(this).data('gnMediaSelector').trigger({type:'addElement.gnMediaSelector'}); 
				}).attr("disabled",true);
				//
				$gnMediaSelector.append(gnAddMedia);
			}
			
			// check if additions are allow and set the "Add Media" buttom appropriately
			checkMultipleSelection($gnMediaSelector);
			
			// if is sortable
			if(event.groupData.sortable)
			{						
				$('.gnms-list', this).sortable({
					//handle: 'div.gnms-handle',
					stop: function(e, ui) {
						$gnMediaSelector.trigger({type:'onOrderChanged.gnMediaSelector'});
					}
				});
				
				$('.gnms-list li',this).each(function(){
					$(this).css('cursor','move');
				})
			}
        });
		
		// for order change
		$gnMediaSelector.bind('addElement.gnMediaSelector', function(event) {
			//
			log($(this).attr('id'));
			//
			$(this).find('ul').append( createMediaElement( $(this), {}) );
			//
			checkMultipleSelection($(this));
        });

		// for order change
		$gnMediaSelector.bind('selectElement.gnMediaSelector', function(event) {
			
			var $fieldData = event.element.data('fieldData');
			
			var $extras = (typeof $fieldData.ID != 'undefined') ?
					 '&#038;ID='+$fieldData.ID+'&#038;srcID='+$fieldData.srcID+'&#038;action=crop':
					 '&#038;action=choose';
			
			if(typeof $fieldData.groupFieldIndex != 'undefined' & $fieldData.groupFieldIndex != -1)
				$extras += '&#038;groupFieldIndex='+$fieldData.groupFieldIndex;

        	tb_show('Media Chooser', userSettings.url+'wp-content/plugins/gn-media-selector/gnms_actions.php?groupName='+opts.groupName+'&#038;postID='+opts.postID+$extras+'&#038;elementIndex='+event.element.index()+'&#038;TB_iframe=1');
		});
		
		// for order change
		$gnMediaSelector.bind('removeElement.gnMediaSelector', function(event) {
			removeMediaElement(event.element ,$(this));			
        });

		// for order change
		$gnMediaSelector.bind('onOrderChanged.gnMediaSelector', function(event) {
			updateMediaOrder($(this));			
        });
		
		
		$gnMediaSelector.updateSelector = function( $groupName, $ID, $elementIndex )
		{
			var element = $('.gnms-list li',$(this)).get($elementIndex);
			updateMediaElement( $(element), $groupName, $ID );
		}
		
		// load our data & intiate
        loadSelectorData($gnMediaSelector, 	opts.groupName, opts.postID);
       			
        return $gnMediaSelector;
    };
	
	// plugin function wrapper
 	$.fn.updateSelector = function( $groupName, $ID, $elementIndex ) { 
		if($(this).data('gnMediaSelector') && $(this).data('gnMediaSelector').updateSelector)
	    $(this).data('gnMediaSelector').updateSelector( $groupName, $ID, $elementIndex );
	};
    

    // plugin wrapper
    $.fn.gnMediaSelector = function(options) {
        return this.each(function() {
            //
            var element = $(this);
            // Return early if this element already has a plugin instance
            if (element.data('gnMediaSelector')) return;
			//
            var gnMediaSelector = $.gnMediaSelector(options, this);
			// Store plugin object in this element's data
            element.data('gnMediaSelector', gnMediaSelector);        });
    };

    // default plugin options
    $.fn.gnMediaSelector.defaults = {
        // do something with the dom object before you render it, add nodes, get more data from the services etc.
        beforeRender: function(tracksData) {
            var $gnMediaSelector = $(this);
        },
        // initialization, when dom is ready
        onDomReady: function() {
            $('div.gnms').gnMediaSelector();
        },
		groupName:null,
		postID: null
    };

    // -------------------------------------------------------------------
    // the default Auto-Initialization
    $(function() {
        if ($.isFunction($.fn.gnMediaSelector.defaults.onDomReady)) {
            	$.fn.gnMediaSelector.defaults.onDomReady();
        }
    });
	
	// set up our ajax if it hasn't been done already
    $.ajaxSetup({
        type: 'POST',
        url: ajaxurl,
        cache: false,
        timeout: (30 * 1000)
    });

})(jQuery);

//
String.prototype.truncate = function(n)
{
	return (this.length > n) ? this.substr(0,n-1)+'...' : this.substr(0,this.length);
}

//
function GNMSUpdateMediaSelector( $groupName, $ID, $elementIndex  )
{
	jQuery('div#'+$groupName).updateSelector( $groupName, $ID, $elementIndex );
}